require 'lib/configfile/postfix'
require 'main/container'

Figlet = :PostfixFiglet

    include ConfigFile::Postfix

    class PostfixFiglet < Core::Figlet
	attr_reader :global_cnf
	attr_reader :aliases_cnf
	attr_reader :virtual_cnf
	attr_reader :vquota_cnf
	attr_reader :mboxsize_cnf
	
	def initialize
	    super("Postfix")
# quota for virtual domains is not supported	    
#	    @filenames = ["main.cf","aliases","virtual","vquota"]
	    @filenames = ["main.cf","aliases","virtual","mboxsize"]
	    @base_path = "/etc/postfix/"
	    @global_cnf = nil
	    @aliases_cnf = nil
	    @virtual_cnf = nil
# quota for virtual domains is not supported	    
#	    @vquota_cnf = nil
	    @mboxsize_cnf = nil
	    loadconfig
	    main
	end
	
	def firewall_interact
	    {"INPUT" => ["-A INPUT -i eth1 -p tcp -m tcp --dport 25 -j ACCEPT","-A INPUT -i eth1 -p tcp -m tcp --dport 110 -j ACCEPT","-A INPUT -i eth1 -p tcp -m tcp --dport 143 -j ACCEPT"],
	     "OUTPUT" => ["-A OUTPUT -o eth1 -p tcp -m tcp --sport 25 -j ACCEPT","-A OUTPUT -o eth1 -p tcp -m tcp --sport 110 -j ACCEPT","-A OUTPUT -o eth1 -p tcp -m tcp --sport 143 -j ACCEPT"]}
	end
	
	def save
	    saveconfig
	    reloadservice
	end

	def load
	    loadconfig
	    reloadservice
	    main
	end

	def reloadservice
	    #Kernel::system_guarded("/usr/sbin/postmap /etc/postfix/virtual")
	    #Kernel::system_guarded("/usr/sbin/postalias /etc/postfix/aliases")
	    #Kernel::system_guarded("/usr/sbin/postmap /etc/postfix/mboxsize")
	    Kernel::service_run(:postfix,:restart)
	end

	def startservice
	    Kernel::service_run(:postfix,:start)
	end

	def stopservice
	    Kernel::service_run(:postfix,:stop)
	end
	
	def loadconfig
	    if File.exist?(@base_path + @filenames[0])
		@global_cnf = ConfigFile::Postfix::PostfixConfig.new(@base_path + @filenames[0])
	    end
	    if File.exist?(@base_path + @filenames[1])
		@aliases_cnf = ConfigFile::Postfix::AliasConfig.new(@base_path + @filenames[1])
	    end
	    if File.exist?(@base_path + @filenames[2])
		@virtual_cnf = ConfigFile::Postfix::VirtualConfig.new(@base_path + @filenames[2])
	    end 
# quota for virtual domains is not supported	    
#	    if File.exist?(@base_path + @filenames[3])
#		@vquota_cnf = ConfigFile::Postfix::VquotaConfig.new(@base_path + @filenames[3])
#	    end 
	    if File.exist?(@base_path + @filenames[3])
		@mboxsize_cnf = ConfigFile::Postfix::MboxsizeConfig.new(@base_path + @filenames[3])
	    end 
	end
	
	def saveconfig
	    File.open(@base_path + @filenames[0], "w") do |file|
		file.print @global_cnf
	    end
	    File.open(@base_path + @filenames[1], "w") do |file|
		file.print @aliases_cnf
	    end
	    File.open(@base_path + @filenames[2], "w") do |file|
		file.print @virtual_cnf
	    end
# quota for virtual domains is not supported	    
#	    File.open(@base_path + @filenames[3], "w") do |file|
#		file.print @vquota_cnf
#	    end
	    File.open(@base_path + @filenames[3], "w") do |file|
		file.print @mboxsize_cnf
	    end
	end
	
	def main
	    @hierarchy = nil
	    ## constructing the hierarchy
	    @hierarchy = Core::Notebook.new("Postfix")
	    @hierarchy.residence = self

    	    @global = GlobalConfig.new('postfix_global', @hierarchy, @global_cnf)
    	    @aliases = AliasList.new('postfix_aliases', @hierarchy, @aliases_cnf)
    	    @virtual = VirtualList.new('postfix_virtual', @hierarchy, @virtual_cnf)
# quota for virtual domains is not supported	    
#    	    @vquota = VquotaList.new('postfix_vquota', @hierarchy, @vquota_cnf)
    	    @mboxsize = MboxsizeList.new('postfix_mboxsize', @hierarchy, @mboxsize_cnf)
	    
	    important_opts = ["mailbox_size_limit","message_size_limit","virtual_mailbox_limit","inet_interfaces","mydestination","myhostname","mynetworks"]
	    
	    @global_cnf.each_value do |v|
		    Core::Leaf.new(v.to_key, @global).set v.option if important_opts.include?(v.to_key)
	    end
	    @aliases_cnf.each_value do |v|
		    Alias.new(v.to_key, @aliases, v).set v.option
	    end
	    @virtual_cnf.each_value do |v|
		   @domain = Virtual.new(v.to_key.strip, @virtual, v)
	    end
# quota for virtual domains is not supported	    
#	    @vquota_cnf.each_value do |v|
#		   Vquota.new(v.to_key, @vquota, v).set v.option
#	    end

	    @mboxsize_cnf.each_value do |v|
		   Mboxsize.new(v.to_key, @mboxsize, v).set v.option
	    end if @mboxsize_cnf
	    
	    Core::Action.new("Write config", @hierarchy) do |x| 
	      save
		@hierarchy
	    end

	    # start and stop service buttons
    	    @status_msg = Core::Text.new("Service status",@hierarchy,"")
    	    if get_status
		@status_str = 'Running...'
	    else
		@status_str = 'Stopped...'
	    end
    	    @stop_button = Core::Action.new("Service stop", @hierarchy) { |x| 
		stopservice
		unless get_status
        	    @status_msg.set 'Stopped...'
		end	
		@hierarchy
	    }
    	    @start_button = Core::Action.new("Service start", @hierarchy) { |x| 
	        startservice
	        if get_status
    		    @status_msg.set 'Running...'
		end	
		@hierarchy
	    }
    	    @status_msg.set @status_str
	end

        def get_status
    	    fname = '/var/lock/subsys/postfix'
    	    File.exist?(fname)
        end	
    end

    class GlobalConfig < Core::Notebook
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
	    Core::Action.new("Write config", self) do |x| 
		x.each {|obj, val|
		obj.set(val)
		@residence.global_cnf.option(obj.name).option = val if obj.class == Core::Leaf
		}
		self
	    end
	end
    end

    class AliasList < Core::MutableList
	attr_reader :options
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent) {|x| Alias===x}
		AddAlias.new("addAlias",self,options)
	    @options = options
	end
    end

    class AddAlias < Core::Container
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
		Core::Leaf.new("alias",self)
		Core::Leaf.new("user",self)
		Core::Action.new("save",self){ |x| addAlias_trigger(x,options) }
	end
	def addAlias_trigger(x,options)
		x.each {|obj, val|
	    	    @alias = val if obj.name == "alias"
    		    @user = val if obj.name == "user"
		}
		@option = ConfigFile::Postfix::AliasOption.new(@alias,@user)	
    		options.add_option(@option)
		Alias.new(@option.to_key, parent,options).set @option.option
		parent
	end
    end

    class Alias < Core::Leaf
	def initialize (name, parent, options)
		@options = options
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
	end
	def actionDelete
	    parent.options.del_option(@options.to_key) unless (@options.class == ConfigFile::Postfix::VirtualConfig || @options.class == ConfigFile::Postfix::AliasConfig) 
#	    parent.delChild(self)
	    parent
	end
    end

    class VirtualList < Core::MutableList
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent) {|x| Virtual===x}
		AddVirtual.new("addVirtual",self,options)
	end
    end

    class AddVirtual < Core::Container
	def initialize (name, parent, options)
		@options = options
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
		Core::Leaf.new("domain",self)
		Core::Action.new("save",self){ |x| addDomain_trigger(x,@options) }
	end
	def addDomain_trigger(x,options)
		x.each {|obj, val|
	    	@domain = val if obj.name == "domain"
			}
		@option = ConfigFile::Postfix::DomainDefinition.new([],@domain)	
		@residence.virtual_cnf.add_option(@option)
		Virtual.new(@domain,parent, options)
		parent
	end
    end

    class Virtual < Core::MutableList
	attr_reader :options
	def initialize (name, parent,options)
		@name = name
		@options = options
		raise "Parent must be not nil" if parent.nil?
		super(name, parent) {|x| Alias===x}
		@options.each_value do |opt| 
			Alias.new(opt.to_key,self,opt).set opt.option if opt.class == ConfigFile::Postfix::VirtualOption
		end
		AddVirtualAlias.new("addAlias",self,@options)
	end
	def actionDelete
    	    @residence.virtual_cnf.del_option(@name)
	    parent.delChild(self)
	    parent
	end
    end

    class AddVirtualAlias < Core::Container
	def initialize (name, parent, options)
		@options = options
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
		Core::Leaf.new("alias",self)
		Core::Leaf.new("user",self)
		Core::Action.new("save",self){ |x| addVirtualAlias_trigger(x,@options) }
	end
	def addVirtualAlias_trigger(x,options)
		x.each {|obj, val|
	    	@alias = val if obj.name == "alias"
			@user = val if obj.name == "user"
			}
		@option = ConfigFile::Postfix::VirtualOption.new(@alias,@user)	
	    @options.add_option(@option)
		Alias.new(@option.to_key, parent, options).set @option.option
		parent
	end
	def actionDelete
	    @residence.cnf.del_option(@options.to_key)
	    oldparent = @parent
	    oldparent.delChild(self)
	    oldparent
	end
    end

# quota for virtual domains is not supported	    
#    class VquotaList < Core::MutableList
#	def initialize (name, parent, options)
#	    raise "Parent must be not nil" if parent.nil?
#	    super(name, parent) {|x| Vquota===x}
#		AddVquota.new("addVquota",self,options)
#	end
#    end
#
#    class AddVquota < Core::Container
#	def initialize (name, parent, options)
#	    raise "Parent must be not nil" if parent.nil?
#	    super(name, parent)
#		Core::Leaf.new("user",self)
#		Core::Leaf.new("vlimit",self)
#		Core::Action.new("save",self){ |x| addVquota_trigger(x,options) }
#	end
#	def addVquota_trigger(x,options)
#		x.each {|obj, val|
#	    	    @vlimit = val if obj.name == "vlimit"
#    		    @user = val if obj.name == "user"
#		}
#		@option = ConfigFile::Postfix::VquotaOption.new(@user,@vlimit)	
#    		options.add_option(@option)
#		Vquota.new(@option.to_key, parent,options).set @option.option
#		parent
#	end
#    end
#
#    class Vquota < Core::Leaf
#	def initialize (name, parent, options)
#	    @options = options
#	    raise "Parent must be not nil" if parent.nil?
#	    super(name, parent)
#	end
#	def actionDelete
#    	    @residence.vquota_cnf.del_option(@name)
#	    parent.delChild(self)
#	    parent
#	end
#    end
#

    class MboxsizeList < Core::MutableList
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent) {|x| Mboxsize===x}
		AddMboxsize.new("addMboxsize",self,options)
	end
    end

    class AddMboxsize < Core::Container
	def initialize (name, parent, options)
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
		Core::Leaf.new("user",self)
		Core::Leaf.new("mbox",self)
		Core::Action.new("save",self){ |x| addMboxsize_trigger(x,options) }
	end
	def addMboxsize_trigger(x,options)
		x.each {|obj, val|
	    	    @mbox = val if obj.name == "mbox"
    		    @user = val if obj.name == "user"
		}
		@option = ConfigFile::Postfix::MboxsizeOption.new(@user,@mbox)	
    		options.add_option(@option)
		Mboxsize.new(@option.to_key, parent,options).set @option.option
		parent
	end
    end

    class Mboxsize < Core::Leaf
	def initialize (name, parent, options)
	    @options = options
	    raise "Parent must be not nil" if parent.nil?
	    super(name, parent)
	end
	def actionDelete
    	    @residence.mboxsize_cnf.del_option(@name)
	    parent.delChild(self)
	    parent
	end
    end

